/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System.Collections;
using System.Web.UI.HtmlControls;
using Com.PingIdentity.Adapters.Pftoken.Core;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;
using System;
using System.Web.UI;
using System.IO;
using System.Configuration;
using System.Web;
using System.Web.Security;

namespace Com.PingIdentity.PingFederate.SampleApp.IdpSample.web {

	/// <summary>
	/// Shows user attributes and connections
	/// </summary>
	public class MainForm : Page {

		public const int MAX_LENGTH = 40;
		protected HtmlAnchor IdpStartSLOUrl;

		private Links samlLinks;

		private void Page_Load(object sender, EventArgs e) {

			//prevent the output of aspx page from being cached by the browser
			Response.AddHeader("Cache-Control","no-cache");
			Response.AddHeader("Pragma","no-cache");

			//get properties from Idp config file
			Hashtable idpConfigTable = (Hashtable)Session[
				Constants.IDP_CONFIG_TABLE];

			samlLinks = new Links(idpConfigTable, Request);

			String idpSLOLink = samlLinks.getIdpSLOLink();
			String errorMessage = "Error+during+idp-initiated+SLO";
			idpSLOLink = samlLinks.addParameter(idpSLOLink, Constants.IN_ERROR_RESOURCE,
				samlLinks.getIdpErrorPageUrl() + errorMessage);
			this.IdpStartSLOUrl.HRef = idpSLOLink;

			Hashtable userInfo = (Hashtable)Session[Constants.USER_INFO];
			if((userInfo == null) || (userInfo.Count <= 0)) {
				String loginPage = samlLinks.getIdpLoginPageUrl(Constants.LOCAL_TYPE);
				Response.Redirect(loginPage);
			}
		}

		protected void ShowSpConnections() {
			//read properties from Idp connections XML file
			String idpConnections  = ConfigurationSettings.AppSettings["connections-file"];
			String connectionsPath = Context.Server.MapPath("config/" + idpConnections);
			if (!File.Exists(connectionsPath)) 
			{
				SSOEntity[] spList = null;
				try 
				{
					SSODirectoryServiceService directoryService = new SSODirectoryServiceService();
					spList = directoryService.getSPList();
				}
				catch(Exception e) {
					// Log exception
					HttpContext.Current.Trace.Warn(e.ToString());
					// Display alert
					Response.Write("<script language=\"javascript\">");
					Response.Write("alert('Note that a connection supporting IdP initiated SSO should be configured before you can perform an SSO from the IdP application. Please configure such a connection and ensure that the PingFederate server is running.')");
					Response.Write("</script>");
					return;
				}
				IEnumerator list = spList.GetEnumerator();
				list.Reset();
				int i=0;
				while(list.MoveNext()) 
				{
					i++;
					SSOEntity sp = (SSOEntity)list.Current;
					String connectionName = (String)sp.entityId;
					String description = "Click on the image to initiate Single sign on";
					String idpSSOUrl = null;
					
					idpSSOUrl = samlLinks.getIdpStartSSOUrl(connectionName);

					Response.Write("<table class=\"connection\">");
					Response.Write("<tr>");
					Response.Write("<td>");
					Response.Write("<div class=\"headeridp\">");
					Response.Write("SP Connection");
					Response.Write("<a href=\"javascript:openHelp('help.html?#sso');\">");
					Response.Write("<img src=\"images/HelpIcon.gif\" border=\"0\">");
					Response.Write("</a>");
					Response.Write("</div>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td align=\"left\">");
					Response.Write("<span class=\"text\"><strong>Connection Id: </strong>");
					Response.Write(connectionName);
					Response.Write("</span></td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td>");
					Response.Write("<span class=\"text\">");
					Response.Write(description);
					Response.Write("</span>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td>");
					Response.Write("<a class=\"image\" href=\"#\">");
					Response.Write("<img id=\"federate");
					Response.Write(i + 1);
					Response.Write("\" name=\"federate");
					Response.Write(i + 1);
					Response.Write("\" border=\"0\"");
					Response.Write("src=\"images/Federate_Normal.gif\"");
					Response.Write("OnMouseOver=\"javascript:document.all['federate");
					Response.Write(i + 1);				
					Response.Write("'].src='images/Federate_Hover.gif';\"");
					Response.Write("OnMouseOut=\"javascript:document.all['federate");
					Response.Write(i + 1);				
					Response.Write("'].src='images/Federate_Normal.gif';\"");
					Response.Write("OnClick=\"javascript:initiateSSO('");
					Response.Write(idpSSOUrl);
					Response.Write("')\">");
					Response.Write("</a>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("</table>");

				}
				// Check to see if any connections were obtained
				if(i ==0 ) 
				{
					Response.Write("<script language=\"javascript\">");
					Response.Write("alert('No SP connections were found, please make sure PingFederate is running and there are connections configured.')");
					Response.Write("</script>");
				}
			}
			else 
			{
				//get properties from Idp connections file
				ArrayList idpConnectionsList = (ArrayList)Session[Constants.IDP_CONNECT_LIST];
				for(int i = 0; i < idpConnectionsList.Count; i++) 
				{
					Hashtable connection  = (Hashtable)idpConnectionsList[i];
					String connectionName = (String)connection[Constants.CONNECTION_NAME];
					String partnerSpId    = (String)connection[Constants.CONNECTION_VALUE];
					String targetResource = (String)connection[Constants.TARGET_RESOURCE];
					String idpAdapterId = (String)connection[Constants.IDP_ADAPTER_ID];
					String description = (String)connection[Constants.DESCRIPTION];
					String idpSSOUrl = null;
					
					//get properties from SP config file
					Hashtable spConfigTable = (Hashtable)Session[Constants.IDP_CONFIG_TABLE];

					//Check to see if IdP Discovery is enabled
					String idpDiscovery = (String)spConfigTable[Constants.IDP_DISCOVERY];
					if(idpDiscovery == Constants.TRUE_STRING_VALUE)
					{
						// Do not send the IdP Adapter Id if IdP discovery is enabled
						idpSSOUrl = samlLinks.getIdpStartSSOUrl(partnerSpId, targetResource);
					}
					else 
					{
						idpSSOUrl = samlLinks.getIdpStartSSOUrl(partnerSpId, targetResource, idpAdapterId);
					}

					String errorMessage   = "Error+during+SSO+to+" + connectionName;
					idpSSOUrl = samlLinks.addParameter(idpSSOUrl, Constants.IN_ERROR_RESOURCE,
						samlLinks.getIdpErrorPageUrl() + errorMessage);

					Response.Write("<table class=\"connection\">");
					Response.Write("<tr>");
					Response.Write("<td>");
					Response.Write("<div class=\"headeridp\">");
					Response.Write("SP Connection");
					Response.Write("<a href=\"javascript:openHelp('help.html?#sso');\">");
					Response.Write("<img src=\"images/HelpIcon.gif\" border=\"0\">");
					Response.Write("</a>");
					Response.Write("</div>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td align=\"left\">");
					Response.Write("<span class=\"text\"><strong>Connection Id: </strong>");
  					Response.Write(partnerSpId);
					Response.Write("</span></td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td align=\"left\">");
					Response.Write("<span class=\"text\"><strong>Description: </strong>");
					Response.Write(description);
					Response.Write("</strong></span>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td>");
					Response.Write("<a class=\"image\" href=\"#\">");
					Response.Write("<img id=\"federate");
					Response.Write(i + 1);
					Response.Write("\" name=\"federate");
					Response.Write(i + 1);
					Response.Write("\" border=\"0\"");
					Response.Write("src=\"images/Federate_Normal.gif\"");
					Response.Write("OnMouseOver=\"javascript:document.all['federate");
					Response.Write(i + 1);				
					Response.Write("'].src='images/Federate_Hover.gif';\"");
					Response.Write("OnMouseOut=\"javascript:document.all['federate");
					Response.Write(i + 1);				
					Response.Write("'].src='images/Federate_Normal.gif';\"");
					Response.Write("OnClick=\"javascript:initiateSSO('");
					Response.Write(idpSSOUrl);
					Response.Write("')\">");
					Response.Write("</a>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td>");
					Response.Write("<span class=\"text\"><strong>Direct Link: </strong><a class=\"text\" href=\"");
					Response.Write(targetResource);				
					Response.Write("\">");
					Response.Write(targetResource);				
					Response.Write("</a></span>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("</table>");
		
				}
			}
		}

		protected void ShowUserAttributes() {

			Hashtable userInfo = (Hashtable)Session[Constants.USER_INFO];
			if(userInfo != null && userInfo.Count > 0) {
				foreach(String key in userInfo.Keys) {
					String infoName = key;
					String infoValue = (String)userInfo[key];
					// Handle Authn Context
					if(infoName == PftokenAgent.AUTHN_CTX_ATTRIBUTE_NAME) 
					{
						infoValue = Constants.DisplayAuthnContext(infoValue);
					}
					// Check if value is too long
					if(infoValue.Length > MAX_LENGTH) 
					{
						infoValue = infoValue.Substring(0,MAX_LENGTH);
						infoValue += "...";
					}
					Response.Write("<tr>");
					Response.Write("<td align=\"left\" width=\"50\">");
					Response.Write("<span class=\"text\">");
					Response.Write("<strong>");
					Response.Write(infoName);
					Response.Write("</strong>");
					Response.Write("</span>");
					Response.Write("</td>");
					Response.Write("<td align=\"left\">");
					Response.Write("<span class=\"text\">");
					Response.Write(infoValue);
					Response.Write("</span>");
					Response.Write("</td>");
					Response.Write("</tr>");

				}
			}
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}